@@ -4,8 +4,8 @@
  * This software is licensed under the terms of the Apache Licence Version 2.0
  * which can be obtained at http://www.apache.org/licenses/LICENSE-2.0.
  * In applying this licence, ECMWF does not waive the privileges and immunities
- * granted to it by virtue of its status as an intergovernmental organisation nor
- * does it submit to any jurisdiction.
+ * granted to it by virtue of its status as an intergovernmental organisation
+ * nor does it submit to any jurisdiction.
  */
 
 /*! \file FortranMagics.cc
@@ -19,15 +19,15 @@
 
 */
 
-
 #include "FortranMagics.h"
-#include "Timer.h"
 
 #include "Coastlines.h"
 #include "LegendVisitor.h"
+#include "MagicsGlobal.h"
 #include "RootSceneNode.h"
 #include "SceneNode.h"
 #include "TextVisitor.h"
+#include "Timer.h"
 #include "ViewNode.h"
 #include "VisualAction.h"
 #ifdef HAVE_GRIB
@@ -39,8 +39,12 @@
 #include "Contour.h"
 #include "GeoPointsDecoder.h"
 #include "GraphPlotting.h"
+#include "ImportAction.h"
+#include "ImportObjectHandler.h"
+#include "ImportPlot.h"
 #include "InputMatrix.h"
 #include "MapGenDecoder.h"
+#include "MetaData.h"
 #include "SimplePolylineInput.h"
 #include "SimplePolylineVisualiser.h"
 #include "SymbolInput.h"
@@ -51,12 +55,6 @@
 #include "Wind.h"
 #include "XYList.h"
 
-#include "ImportAction.h"
-#include "ImportObjectHandler.h"
-#include "ImportPlot.h"
-
-#include "MetaData.h"
-
 #ifdef HAVE_NETCDF
 #include "NetcdfDecoder.h"
 #endif
@@ -67,7 +65,6 @@
 #endif
 
 #include "ObsJSon.h"
-
 #include "ObsPlotting.h"
 
 using namespace magics;
@@ -88,7 +85,6 @@ FortranMagics::FortranMagics() :
     singleton_ = this;
 }
 
-
 FortranMagics::~FortranMagics() {
     if (drivers_)
         delete drivers_;
@@ -115,8 +111,9 @@ void FortranMagics::print(ostream& out) const {
 
 void FortranMagics::popen() {
     MagLog::info() << "popen()" << endl;
-    if (getEnvVariable("MAGPLUS_QUIET").empty() && !silent_) {
-        MagLog::userInfo() << "------------------------------------------------------------------\n";
+    if (getEnvVariable("MAGPLUS_QUIET").empty() && !MagicsGlobal::silent()) {
+        MagLog::userInfo() << "----------------------------------------------------"
+                              "--------------\n";
         MagLog::userInfo() << "\n";
         MagLog::userInfo() << "			  " << getMagicsVersionString() << "\n";
         MagLog::userInfo() << "\n";
@@ -126,9 +123,11 @@ void FortranMagics::popen() {
         MagLog::userInfo() << "\n";
         MagLog::userInfo() << "   The European Centre for Medium-Range Weather Forecasts\n";
         MagLog::userInfo() << "\n";
-        // MagLog::userInfo() << "		      Copyright ECMWF "<<MAGICS_COPYRIGHT_PERIOD<<"\n";
+        // MagLog::userInfo() << "		      Copyright ECMWF
+        // "<<MAGICS_COPYRIGHT_PERIOD<<"\n";
         MagLog::userInfo() << "\n";
-        MagLog::userInfo() << "------------------------------------------------------------------\n";
+        MagLog::userInfo() << "----------------------------------------------------"
+                              "--------------\n";
     }
     // actions_.push(&FortranMagics::legend);
     actions_.push(&FortranMagics::subpage);
@@ -173,37 +172,29 @@ int FortranMagics::pclose() {
         return -1;
     }
 
-
-    // the Magics log messages are not broadcast until the next log event - therefore, the
-    // last log message will not be broadcast. We fix that by flushing the message streams
-    // - we only need to do one of them, and all will be flushed behind the scenes.
+    // the Magics log messages are not broadcast until the next log event -
+    // therefore, the last log message will not be broadcast. We fix that by
+    // flushing the message streams
+    // - we only need to do one of them, and all will be flushed behind the
+    // scenes.
     MagLog::info().flush();
 
     // We reset all the parameters to their default,
-    // then a consecutive call to popen will not be affected by the current values.
+    // then a consecutive call to popen will not be affected by the current
+    // values.
     ParameterManager::reset();
 
     Layout::reset();
 
-    if (getEnvVariable("MAGPLUS_QUIET").empty() && !silent_) {
-        /*
-            MagLog::userInfo() << "------------------------------------------------------------------\n";
-            MagLog::userInfo() << " Output files generated:\n";
-            stringarray::iterator it = output_resource_list_.begin();
-            stringarray::iterator itend = output_resource_list_.end();
-            for(; it != itend; it++)
-            {
-                MagLog::userInfo() << "  - "<<(*it)<<"\n";
-            }
-
-            MagLog::userInfo() << "\n";
-        */
-        MagLog::userInfo() << "------------------------------------------------------------------\n";
+    if (getEnvVariable("MAGPLUS_QUIET").empty() && !MagicsGlobal::silent()) {
+        MagLog::userInfo() << "----------------------------------------------------"
+                              "--------------\n";
         MagLog::userInfo() << "    COMPLETED\n";
         MagLog::userInfo() << "\n";
         MagLog::userInfo() << "    Any problems or suggestions? Please contact us at\n";
         MagLog::userInfo() << "                   magics@ecmwf.int\n";
-        MagLog::userInfo() << "------------------------------------------------------------------\n";
+        MagLog::userInfo() << "----------------------------------------------------"
+                              "--------------\n";
     }
     singleton_ = 0;
     return 0;
@@ -228,7 +219,6 @@ void FortranMagics::subpage() {
     push(axisContainer_);
 }
 
-
 void FortranMagics::page() {
     if (empty())
         return;
@@ -239,7 +229,6 @@ void FortranMagics::page() {
             break;
     }
 
-
     FortranSceneNode* node = new FortranSceneNode();
     root_->insert(node);
     // Just a test for metadata collection
@@ -248,7 +237,6 @@ void FortranMagics::page() {
     push(node);
 }
 
-
 void FortranMagics::newpage() {
     if (empty())
         return;
@@ -266,7 +254,6 @@ void FortranMagics::newpage() {
     // push(node);
 }
 
-
 void FortranMagics::superpage() {
     root_ = new FortranRootSceneNode();
 
@@ -278,7 +265,6 @@ void FortranMagics::superpage() {
     drivers_->openDrivers();
 }
 
-
 void FortranMagics::simplelegend() {
     // used fronm the python interface!
     // add a new legend!
@@ -304,7 +290,6 @@ void FortranMagics::legend() {
         legends_.push_back(new FortranAutomaticLegendVisitor());
 }
 
-
 void FortranMagics::plegend() {
     legend_todo_ = true;
     legends_.clear();
@@ -314,7 +299,6 @@ void FortranMagics::plegend() {
 void FortranMagics::poverlay() {
     actions();
 
-
     action_             = new VisualAction();
     ImportAction* input = new ImportAction();
     ImportPlot* plot    = new ImportPlot();
@@ -324,7 +308,6 @@ void FortranMagics::poverlay() {
     action_->visdef(plot);
 }
 
-
 void FortranMagics::pimport() {
     actions();
     ImportObjectHandler* object = new ImportObjectHandler();
@@ -334,7 +317,6 @@ void FortranMagics::pimport() {
         top()->push_back(object);
 }
 
-
 void FortranMagics::pnew(const string& type) {
     MagLog::info() << "pnew(" << type << ")" << endl;
 
@@ -373,13 +355,11 @@ void FortranMagics::pnew(const string& type) {
     axisContainer_ = 0;
     action_        = 0;
 
-
     string legend;
     ParameterManager::get("legend", legend);
     legend_todo_ = magCompare(legend, "on");
 }
 
-
 void FortranMagics::actions() {
     Timer timer("magics", "setting");
     while (!actions_.empty()) {
@@ -390,7 +370,6 @@ void FortranMagics::actions() {
     }
 }
 
-
 void FortranMagics::pcoast() {
     actions();
 
@@ -424,18 +403,15 @@ void FortranMagics::pobs() {
     }
 #endif
 
-
     action_ = new VisualAction();
     action_->data(new ObsJSon());
     top()->push_back(action_);
     action_->visdef(new ObsPlotting());
     return;
 
-
     MagLog::warning() << "No Support for Obs Plotting" << endl;
 }
 
-
 #include "MatrixTestDecoder.h"
 void FortranMagics::ptest() {
     actions();
@@ -444,7 +420,6 @@ void FortranMagics::ptest() {
     top()->push_back(action_);
 }
 
-
 /*!  \brief Finish plot by checking axis, legend and texts
  *
  */
@@ -480,7 +455,6 @@ void FortranMagics::finish() {
     texts_.clear();
 }
 
-
 /*!  \brief Dispatch scene graph to drivers
  * \sa DriverManager::dispatch()
  * \sa FortranRootSceneNode::visualise()
@@ -493,7 +467,6 @@ void FortranMagics::dispatch() {
     root_->release();
 }
 
-
 void FortranMagics::pmapgen() {
     actions();
 
@@ -512,7 +485,6 @@ const char* FortranMagics::metagrib() {
     MetaDataCollector request, needAttributes;
     MagDef attributes;
 
-
     library->askId(request);
     grib.visit(request);
 
@@ -534,6 +506,33 @@ const char* FortranMagics::metagrib() {
     return temp.c_str();
 }
 
+const char* FortranMagics::knownDrivers() {
+    vector<string> drivers;
+
+
+    OutputHandler::drivers(drivers);
+
+    ostringstream out;
+
+    out << "{ \"drivers\" : [";
+
+    string sep = "";
+
+    for (auto& d : drivers) {
+        out << sep << "\"" << d << "\"";
+
+        sep = ", ";
+    }
+    out << "]}";
+
+
+    static string results;
+    results = out.str();
+
+
+    return results.c_str();
+}
+
 #include "MagConfig.h"
 
 const char* FortranMagics::detect(const string& data, const string& dim) {
@@ -568,7 +567,6 @@ const char* FortranMagics::detect(const string& data, const string& dim) {
     return empty.c_str();
 }
 
-
 const char* FortranMagics::metanetcdf() {
 #ifdef HAVE_NETCDF
     NetcdfDecoder netcdf;
@@ -622,8 +620,7 @@ void FortranMagics::pgrib() {
     int index;
     ParameterManager::get("grib_field_position", index);
 
-
-    if (grib == gribfile) {
+    if (grib == gribfile && MagicsGlobal::compatibility()) {
         if (index == gribindex_) {
             gribindex_++;
         }
@@ -693,7 +690,6 @@ void FortranMagics::podb() {
     actions();
 #ifdef HAVE_ODB
 
-
     action_ = new VisualAction();
     action_->data(new OdaGeoDecoder());
     top()->push_back(action_);
@@ -701,7 +697,6 @@ void FortranMagics::podb() {
 #endif
 }
 
-
 void FortranMagics::data(Data* data) {
     ASSERT(action_ == 0);
     action_ = new VisualAction();
@@ -709,7 +704,6 @@ void FortranMagics::data(Data* data) {
     top()->push_back(action_);
 }
 
-
 bool FortranMagics::geographical() {
     string projection;
     ParameterManager::get("subpage_map_projection", projection);
@@ -730,52 +724,6 @@ void param(const string& from, const string& to, T& val) {
     ParameterManager::set(to, val);
 }
 
-
-void split(VisualAction& action) {
-    /*	string split;
-        //ParameterManager::get("contour_line_plotting", split);
-        if ( magCompare(split, "split"))
-        {
-                MagLog::warning() << " contour_line_plotting is deprecated" << endl;
-                double level;
-                ParameterManager::get("contour_split_level", level);
-                double max, min;
-                ParameterManager::get("contour_max_level", max);
-                ParameterManager::get("contour_min_level", min);
-                // set the below contour
-                ParameterManager::set("contour_max_level", level);
-
-                LineStyle style, highlightstyle;
-                double thickness, highlightthickness;
-                string colour, highlightcolour;
-
-                param("contour_below_line_style", "contour_line_style", style);
-                param("contour_below_line_thickness", "contour_line_thickness", thickness);
-                param("contour_below_line_colour", "contour_line_colour", colour);
-                param("contour_below_highlight_style", "contour_highlight_line_style", highlightstyle);
-                param("contour_below_highlight_thickness", "contour_highlight_line_thickness", highlightthickness);
-                param("contour_below_highlight_colour", "contour_highlight_line_colour",highlightcolour);
-
-                action.visdef(new Contour());
-
-                // set the above contour
-                ParameterManager::set("contour_max_level", max);
-                ParameterManager::set("contour_min_level", level);
-                param("contour_above_line_style", "contour_line_style", style);
-                param("contour_above_line_thickness", "contour_line_thickness", thickness);
-                param("contour_above_line_colour", "contour_line_colour", colour);
-                param("contour_above_highlight_style", "contour_highlight_line_style", style);
-                param("contour_above_highlight_thickness", "contour_highlight_line_thickness", thickness);
-                param("contour_above_highlight_colour", "contour_highlight_line_colour", colour);
-                action.visdef(new Contour());
-                // Should we reset???
-        }
-        else {   */
-    action.visdef(new Contour());
-    //	}
-}
-
-
 void FortranMagics::pcont() {
     // First check any split contour! I hate it!
     Timer timer("pcont", "setting");
@@ -790,7 +738,8 @@ void FortranMagics::pcont() {
         else {
             delete input;
 #ifdef HAVE_GRIB
-            // Sylvie: Is this causing GribDecoder MagExceptions when matrx input is faulty?
+            // Sylvie: Is this causing GribDecoder MagExceptions when matrx input is
+            // faulty?
             action_->data(new GribDecoder());
 #else
             MagLog::warning() << " Attempt to decode data from GRIB, but GRIB support is disabled!" << endl;
@@ -799,7 +748,7 @@ void FortranMagics::pcont() {
         top()->push_back(action_);
     }
 
-    split(*action_);
+    action_->visdef(new Contour());
 }
 
 void FortranMagics::pwind() {
@@ -816,7 +765,8 @@ void FortranMagics::pwind() {
         else {
             delete input;
 #ifdef HAVE_GRIB
-            // Sylvie: Is this causing GribDecoder MagExceptions when matrx input is faulty?
+            // Sylvie: Is this causing GribDecoder MagExceptions when matrx input is
+            // faulty?
             GribDecoder* grib = new GribDecoder();
             grib->dimension(2);
             action_->data(grib);
@@ -831,7 +781,6 @@ void FortranMagics::pwind() {
     action_ = 0;
 }
 
-
 void FortranMagics::ptext() {
     string mode;
     ParameterManager::get("text_mode", mode);
@@ -846,7 +795,6 @@ void FortranMagics::ptext() {
     empty_ = false;
 }
 
-
 void FortranMagics::psymb() {
     actions();
 
@@ -891,7 +839,6 @@ void FortranMagics::psymb() {
 void FortranMagics::pline() {
     actions();
 
-
     if (!action_ || polyinput_todo_) {
         action_ = new VisualAction();
 
@@ -908,7 +855,6 @@ void FortranMagics::pline() {
 void FortranMagics::geojson() {
     actions();
 
-
     action_ = new VisualAction();
 
     GeoJSon* geo = new GeoJSon();
@@ -922,7 +868,6 @@ void FortranMagics::geojson() {
 void FortranMagics::wrepjson() {
     actions();
 
-
     action_ = new VisualAction();
 
     WrepJSon* wrep = new WrepJSon();
@@ -1026,7 +971,6 @@ void FortranMagics::epswind() {
     action_->visdef(epswind);
 }
 
-
 void FortranMagics::epsbar() {
     EpsBar* epsbar = new EpsBar();
     action_->visdef(epsbar);
@@ -1042,6 +986,7 @@ void FortranMagics::epsshading() {
 }
 
 void FortranMagics::paxis() {
+    actions();
     try {
         string orientation;
 
@@ -1050,12 +995,12 @@ void FortranMagics::paxis() {
         if (magCompare(orientation, "vertical")) {
             Axis* vaxis = new VerticalAxis();
             MagLog::dev() << *vaxis << "\n";
-            axis_.push(vaxis);
+            top()->push_back(vaxis);
         }
         else {
             Axis* haxis = new HorizontalAxis();
             MagLog::dev() << *haxis << "\n";
-            axis_.push(haxis);
+            top()->push_back(haxis);
         }
     }
     catch (MagicsException& e) {
@@ -1071,10 +1016,8 @@ void FortranMagics::prepare() {
 void FortranMagics::pgraph() {
     actions();
 
-
     GraphPlotting* graph = new GraphPlotting();
 
-
     // do we need a new Action?
     // we check the previous one!
     // if it looks ok we create a new one
@@ -1087,7 +1030,6 @@ void FortranMagics::pgraph() {
         MagLog::dev() << *input << "\n";
     }
 
-
     action_->visdef(graph);
 }
 
@@ -1110,7 +1052,6 @@ void FortranMagics::ptile() {
     top()->push_back(action_);
 }
 
-
 void FortranMagics::pboxplot() {
     actions();
 
@@ -1125,3 +1066,4 @@ void FortranMagics::pboxplot() {
 }
 
 FortranMagics* FortranMagics::singleton_ = 0;
+MagicsGlobal* MagicsGlobal::singleton_   = 0;
