@@ -26,10 +26,7 @@
 
 */
 
-
 #include "LegendVisitor.h"
-#include "Polyline.h"
-#include "Text.h"
 
 #include "AnimationRules.h"
 #include "Arrow.h"
@@ -38,7 +35,9 @@
 #include "LegendMethod.h"
 #include "MagicsFormat.h"
 #include "MetaData.h"
+#include "Polyline.h"
 #include "Symbol.h"
+#include "Text.h"
 #include "Transformation.h"
 
 using namespace magics;
@@ -67,7 +66,6 @@ LegendVisitor::LegendVisitor() {
     font_size_                           = 0.2;
 }
 
-
 LegendVisitor::~LegendVisitor() {}
 
 /*!
@@ -79,10 +77,8 @@ void LegendVisitor::print(ostream& out) const {
     out << "]";
 }
 
-
 void LegendVisitor::visit(BasicGraphicsObjectContainer&) {}
 
-
 void LegendVisitor::visit(BasicSceneObject& parent) {
     // First visit the parent to collect the legend entry!
     parent.visit(*this);
@@ -168,7 +164,6 @@ void DoubleLineEntry::columnBox(const PaperPoint& point, BasicGraphicsObjectCont
 void LegendVisitor::build() {
     legend_ = new LegendLayout();
 
-
     legend_->x(view_x_);
     legend_->y(view_y_);
     legend_->width(view_width_);
@@ -181,7 +176,6 @@ void LegendVisitor::build() {
 
     grid();
 
-
     // populate the layout!
 
     vector<string>::const_iterator label        = lines_.begin();
@@ -216,7 +210,6 @@ void LegendVisitor::build() {
         back()->units(units_text_);
     }
 
-
     for (auto& entry : *this) {
         entry->width(text_width_);
         Text* legend = new Text();
@@ -264,7 +257,6 @@ void LegendVisitor::build() {
         (magCompare(direction_, "column")) ? (*method_).column(*entry, position->x(), position->y(), *legend, *legend_)
                                            : (*method_).row(*entry, position->x(), position->y(), *legend, *legend_);
 
-
         ++position;
         legend->setAngle((orientation_ / 180.) * 3.14);
         legend_->push_back(legend);
@@ -272,7 +264,6 @@ void LegendVisitor::build() {
     current_->frameIt();
 }
 
-
 void LegendVisitor::horizontal() {
     // Here we have an horizontal legend
     int rows   = 1;
@@ -296,7 +287,8 @@ void LegendVisitor::horizontal() {
         (this->*titleBuilders_[title_position_])();
     }
 
-    // here we have a title, we need to adjust the coordinates used in the legend dependind on its position
+    // here we have a title, we need to adjust the coordinates used in the legend
+    // dependind on its position
     if (magCompare(entry_orientation_, "right_left")) {
         std::reverse(begin(), end());
         for (auto& entry : *this)
@@ -415,7 +407,6 @@ void LegendVisitor::leftTitle() {
     legend_->minX(newminx);
 }
 
-
 void LegendVisitor::grid() {
     if (builders_.empty()) {
         builders_["column"] = &LegendVisitor::vertical;
@@ -434,7 +425,6 @@ void LegendVisitor::grid() {
             direction_ = "column";
     }
 
-
     vector<string>::const_iterator label = lines_.begin();
     entriesNumber_                       = 0;
     for (auto& entry : *this) {
@@ -489,7 +479,6 @@ void SymbolEntry::rowBox(const PaperPoint& point, BasicGraphicsObjectContainer&
     double incx = width / 3;
     double incy = height / 6;
 
-
     Polyline* frame = new Polyline();
     Colour colour   = (borderColour_.automatic()) ? symbol_->getColour() : borderColour_;
     frame->setColour(Colour("grey"));
@@ -515,7 +504,6 @@ void SymbolEntry::columnBox(const PaperPoint& point, BasicGraphicsObjectContaine
     double incx = width / 3;
     double incy = height / 3;
 
-
     Polyline* frame = new Polyline();
     Colour colour   = (borderColour_.automatic()) ? symbol_->getColour() : borderColour_;
     frame->setColour(Colour("grey"));
@@ -562,7 +550,6 @@ void ArrowEntry::columnBox(const PaperPoint& point, BasicGraphicsObjectContainer
     text->setJustification(MLEFT);
     legend.push_back(text);
 
-
     pos.y_ -= 0.2;
     arrow_->push_back(ArrowPoint(arrow_->getScale(), 0, pos));
     legend.push_back(arrow_);
@@ -629,11 +616,9 @@ void LegendEntry::rowBox(const PaperPoint& point, BasicGraphicsObjectContainer&
     frame->push_back(PaperPoint(x + 2, y - height));
     frame->push_back(PaperPoint(x - width, y - height));
 
-
     legend.push_back(frame);
 }
 
-
 void LegendEntry::columnBox(const PaperPoint& point, BasicGraphicsObjectContainer& legend) {
     Polyline* box = new Polyline();
     Colour col    = colour();
@@ -663,14 +648,12 @@ void LegendEntry::columnBox(const PaperPoint& point, BasicGraphicsObjectContaine
     frame->push_back(PaperPoint(x, y - height));
     frame->push_back(PaperPoint(x - width, y - height));
 
-
     legend.push_back(frame);
 
     ostringstream top, bottom;
     top << MagicsFormat(format_, from_);
     bottom << MagicsFormat(format_, to_);
 
-
     Text* from = new Text();
     Text* to   = new Text();
     from->setVerticalAlign(MHALF);
@@ -705,16 +688,13 @@ void BoxEntry::set(const PaperPoint& point, BasicGraphicsObjectContainer& legend
     box_->push_back(PaperPoint(x - width, y - height));
     box_->setColour(Colour("black"));
 
-
     LegendVisitor::addLegendInfo("legend_entry_colour", box_->getFillColour().name());
     LegendVisitor::addLegendInfo("legend_entry_min_text", from());
     LegendVisitor::addLegendInfo("legend_entry_max_text", to());
 
-
     legend.push_back(box_);
 }
 
-
 void BoxEntry::rowBox(const PaperPoint& point, BasicGraphicsObjectContainer& legend) {
     double width  = 1;
     double height = 0.4;
@@ -727,8 +707,9 @@ void BoxEntry::rowBox(const PaperPoint& point, BasicGraphicsObjectContainer& leg
         from->setVerticalAlign(MBOTTOM);
         from->setAngle(angle_);
         legend.push_back(from);
+
         if (automatic_) {
-            if (minText_.empty()) {
+            if (!userMin_) {
                 ostringstream bottom;
                 bottom << MagicsFormat(format_, from_);
                 minText_ = bottom.str();
@@ -738,14 +719,10 @@ void BoxEntry::rowBox(const PaperPoint& point, BasicGraphicsObjectContainer& leg
         }
         else if (!last_)
             from->addText(userText_, font_);
-        else if (last_ && maxText_.size())
-            from->addText("", font_);
-        else {
-            if (automatic_ || maxText_.size()) {
-                ostringstream bottom;
-                bottom << MagicsFormat(format_, from_);
-                from->addText(bottom.str(), font_);
-            }
+        else if (userMax_) {
+            ostringstream bottom;
+            bottom << MagicsFormat(format_, from_);
+            from->addText(bottom.str(), font_);
         }
     }
     if (last_) {
@@ -755,7 +732,7 @@ void BoxEntry::rowBox(const PaperPoint& point, BasicGraphicsObjectContainer& leg
         to->push_back(PaperPoint(x + width, y - height - 0.25));
         legend.push_back(to);
         if (automatic_) {
-            if (maxText_.empty()) {
+            if (!userMax_) {
                 ostringstream to;
                 to << MagicsFormat(format_, to_);
                 maxText_ = to.str();
@@ -766,7 +743,6 @@ void BoxEntry::rowBox(const PaperPoint& point, BasicGraphicsObjectContainer& leg
             to->addText(userText_, font_);
     }
 
-
     Polyline* top = new Polyline();
     top->push_back(PaperPoint(x - (width * 1.), y + (height * 2)));
     top->push_back(PaperPoint(x + (width * 1.), y + (height * 2)));
@@ -788,14 +764,12 @@ void BoxEntry::rowBox(const PaperPoint& point, BasicGraphicsObjectContainer& leg
     // Small check
     Colour colour = (borderColour_.automatic()) ? box_->getFillColour() : borderColour_;
 
-
     if (box_->getFillColour() == Colour("none")) {
         box_->setFilled(false);
         colour = Colour("black");
     }
     box_->setColour(colour);
 
-
     legend.push_back(box_);
     legend.push_back(top);
     legend.push_back(bottom);
@@ -914,7 +888,6 @@ void BoxEntry::rowHisto(const PaperPoint& point, BasicGraphicsObjectContainer& l
         legend.push_back(to);
     }
 
-
     box_->push_back(PaperPoint(x - width, y - bottom));
     box_->push_back(PaperPoint(x - width, y - top));
     box_->push_back(PaperPoint(x + width, y - top));
@@ -954,7 +927,6 @@ void BoxEntry::rowHisto(const PaperPoint& point, BasicGraphicsObjectContainer& l
     bottombox->push_back(PaperPoint(x + width, y + 0.85));
     legend.push_last(bottombox);
 
-
     Polyline* topline = new Polyline();
     topline->setColour(box_->getFillColour());
     topline->setLineStyle(M_DASH);
@@ -968,7 +940,6 @@ void BoxEntry::rowHisto(const PaperPoint& point, BasicGraphicsObjectContainer& l
     bottomline->setThickness(grid_thickness);
     bottomline->setLineStyle(grid_style);
 
-
     bottomline->push_back(PaperPoint(x - width, y + 0.8));
     bottomline->push_back(PaperPoint(x + width, y + 0.8));
     legend.push_last(bottomline);
@@ -1003,7 +974,7 @@ void BoxEntry::columnBox(const PaperPoint& point, BasicGraphicsObjectContainer&
         from->setJustification(MLEFT);
         from->setVerticalAlign(MHALF);
         if (automatic_) {
-            if (minText_.empty()) {
+            if (!userMin_) {
                 ostringstream bottom;
                 bottom << MagicsFormat(format_, from_);
                 minText_ = bottom.str();
@@ -1014,7 +985,7 @@ void BoxEntry::columnBox(const PaperPoint& point, BasicGraphicsObjectContainer&
         else if (!last_)
             from->addText(userText_, font_);
         else {
-            if (automatic_ || maxText_.size()) {
+            if (automatic_ || userMax_) {
                 ostringstream bottom;
                 bottom << MagicsFormat(format_, from_);
                 from->addText(bottom.str(), font_);
@@ -1226,7 +1197,6 @@ void LegendVisitor::getReady() {
         composition_ = "user_text_only";
 }
 
-
 void XmlLegendVisitor::getReady() {
     ASSERT(BasicSceneObject::parent_);
 
@@ -1249,7 +1219,6 @@ void XmlLegendVisitor::getReady() {
     double text_y = bottom.percent() + yb * 100 / BasicSceneObject::parent_->absoluteHeight();
     ;
 
-
     view_x_      = ml.percent();
     view_y_      = mb.percent();
     view_height_ = 100 - mb.percent() - mt.percent();
@@ -1264,7 +1233,6 @@ void XmlLegendVisitor::getReady() {
     Dimension text(font_dimension_, height.absolute(), 10);
     font_size_ = text.absolute();
 
-
     layout_->frame(XmlBasicNodeAttributes::blanking_, XmlBasicNodeAttributes::border_,
                    *XmlBasicNodeAttributes::border_colour_, XmlBasicNodeAttributes::border_style_,
                    XmlBasicNodeAttributes::border_thickness_, Colour("white"));
@@ -1291,7 +1259,6 @@ void FortranPositionalLegendVisitor::getReady() {
     layout_->Layout::frame(blanking_, border_, *border_colour_, border_line_style_, border_thickness_, Colour("white"));
 }
 
-
 void LegendVisitor::finish(BasicGraphicsObjectContainer& parent) {
     newLayout();
     parent.push_back(current_);
@@ -1331,15 +1298,14 @@ void SimpleSymbolEntry::set(const PaperPoint& point, BasicGraphicsObjectContaine
     legend.push_back(symbol_);
 }
 
-
 const string& LegendEntry::label() const {
     if (!label_.empty() || !fromto_)
         return label_;
-    if (minText_.size()) {
+    if (userMin_) {
         label_ = minText_;
         return label_;
     }
-    if (maxText_.size()) {
+    if (userMax_) {
         label_ = maxText_;
         return label_;
     }
@@ -1395,7 +1361,6 @@ void SimpleSymbolEntry::rowBox(const PaperPoint& point, BasicGraphicsObjectConta
             to->addText(userText_, font_);
     }
 
-
     box->push_back(PaperPoint(x - width, y - height));
     box->push_back(PaperPoint(x - width, y + height + height));
     box->push_back(PaperPoint(x + width, y + height + height));
@@ -1410,7 +1375,6 @@ void SimpleSymbolEntry::rowBox(const PaperPoint& point, BasicGraphicsObjectConta
     }
     box->setColour(colour);
 
-
     legend.push_back(box);
 }
 void SimpleSymbolEntry::columnBox(const PaperPoint& point, BasicGraphicsObjectContainer& legend) {
@@ -1484,7 +1448,6 @@ void LegendVisitor::visit(MetaDataVisitor& visitor) {
         c1 = ",";
     }
 
-
     out << c1 << "\"legend_entries\" : [";
     c1 = "";
     for (auto& entry : legendEntriesInfo_) {
