@@ -104,6 +104,7 @@ XmlMagics::XmlMagics() : root_(0), gribloop_(0), geographical_(true), driversToS
     actions_["contour"]   = &XmlMagics::contour;
     actions_["histogram"] = &XmlMagics::histogram;
     actions_["legend"]    = &XmlMagics::legend;
+    actions_["zlegend"]   = &XmlMagics::legend;
     actions_["meta"]      = &XmlMagics::metadata;
 
     actions_["drivers"] = &XmlMagics::driver;
@@ -475,6 +476,7 @@ void XmlMagics::grib(const XmlNode& node) {
         TileDecoder* tile = new TileDecoder();
         tile->set(node);
         if (tile->ok()) {
+            // string tile = node.getAttribute("tile");
             top()->data(tile);
             return;
         }
