@@ -4,8 +4,8 @@
  * This software is licensed under the terms of the Apache Licence Version 2.0
  * which can be obtained at http://www.apache.org/licenses/LICENSE-2.0.
  * In applying this licence, ECMWF does not waive the privileges and immunities
- * granted to it by virtue of its status as an intergovernmental organisation nor
- * does it submit to any jurisdiction.
+ * granted to it by virtue of its status as an intergovernmental organisation
+ * nor does it submit to any jurisdiction.
  */
 
 /*! \file Proj4Projection.cc
@@ -25,7 +25,9 @@
 #include <Polyline.h>
 #include <Proj4Projection.h>
 #include <Text.h>
+
 #include <cmath>
+
 #include "MagConfig.h"
 #include "MetaData.h"
 
@@ -77,8 +79,8 @@ public:
         ostringstream def;
         def << "+proj=geos +h=42164000 +ellps=WGS84 +lon_0=" << from.vertical_longitude_;
 
-        // def << "++proj=tpers    +ellps=WGS84 +h=5000000  +lat_0=20 +lon_0=-60 +x_0=0 +y_0=0 +azi=20 +tilt=0
-        // +units=m";
+        // def << "++proj=tpers    +ellps=WGS84 +h=5000000  +lat_0=20 +lon_0=-60
+        // +x_0=0 +y_0=0 +azi=20 +tilt=0 +units=m";
         definition_ = def.str();
     }
 
@@ -86,19 +88,23 @@ public:
         ostringstream def;
 
         def << "+proj=tpers +ellps=WGS84 +h=" << from.projection_height_;
-        def << "  +lat_0=" << from.projection_view_latitude_;
+        def << " +lat_0=" << from.projection_view_latitude_;
         def << " +lon_0=" << from.projection_view_longitude_;
         def << " +x_0=0 +y_0=0 +azi=" << from.projection_azimuth_;
         def << " +tilt=" << from.projection_tilt_ << "  +units=m";
+
+
         definition_ = def.str();
+        // definition_ = "+proj=tpers +h=5500000 +lat_0=40";
     }
 
     void polarinit(const Proj4Projection& from) {
         ostringstream def;
 
-        def << "+proj=stere +lat_0=90 +lat_ts=" << from.true_scale_north_;
+        def << "+proj=stere +lat_0=90 ";
         def << " +lon_0=" << from.vertical_longitude_;
-        def << " +k=0.994 +x_0=2000000 +y_0=2000000 +ellps=WGS84 +datum=WGS84 +units=m +no_defs";
+        def << " +k=0.994 +x_0=2000000 +y_0=2000000  "
+               "+units=m";
         definition_ = def.str();
     }
 
@@ -107,11 +113,11 @@ public:
 
         def << "+proj=stere +lat_0=-90 +lat_ts=" << from.true_scale_south_;
         def << " +lon_0=" << from.vertical_longitude_;
-        def << " +k=0.994 +x_0=2000000 +y_0=2000000 +ellps=WGS84 +datum=WGS84 +units=m +no_defs";
+        def << "  +x_0=2000000 +y_0=2000000 +ellps=WGS84 +datum=WGS84 "
+               "+units=m ";
         definition_ = def.str();
     }
 
-
     double minlon_;
     double minlat_;
     double maxlon_;
@@ -138,7 +144,6 @@ public:
     const char* definition() { return definition_.c_str(); }
 };
 
-
 class EpsgConfig : public MagConfig {
 public:
     EpsgConfig() {}
@@ -190,7 +195,8 @@ Proj4Projection::Proj4Projection(const string& definition) :
     gridMinLat_(DBL_MAX),
     gridMaxLon_(-DBL_MAX),
     gridMaxLat_(-DBL_MAX),
-    wraparound_(false) {
+    wraparound_(false),
+    helper_(0) {
     // init();
     EpsgConfig config;
     config.init();
@@ -201,7 +207,8 @@ Proj4Projection::Proj4Projection() :
     gridMinLat_(DBL_MAX),
     gridMaxLon_(-DBL_MAX),
     gridMaxLat_(-DBL_MAX),
-    wraparound_(false) {
+    wraparound_(false),
+    helper_(0) {
     // init();
     EpsgConfig config;
     config.init();
@@ -215,7 +222,6 @@ void Proj4Projection::populate(double lon, double lat, double val, vector<UserPo
         out.push_back(UserPoint(lon + 360., lat, val));
 }
 
-
 /*!
   \brief Destructor
 */
@@ -238,17 +244,15 @@ magics::Polyline& Proj4Projection::getUserBoundingBox() const {
 void Proj4Projection::init() {
     MagLog::dev() << "Proj4Projection::init()" << *this << endl;
 
-    from_       = pj_init_plus("+proj=longlat +ellps=WGS84 +datum=WGS84");
     projection_ = Epsg::find(*this);
-    to_         = pj_init_plus(projection_->definition());
+    helper_     = new LatLonProjP(projection_->definition());
 
-    if (!to_) {
+    if (!helper_->valid()) {
         // MagLog::error() << pj_strerrno(pj_errno) << endl;
         MagLog::error() << " proj4 error " << projection_->definition() << endl;
         ASSERT(false);
     }
 
-
     methods_["geos"]   = &Proj4Projection::geos;
     methods_["tpers"]  = &Proj4Projection::tpers;
     methods_["polar"]  = &Proj4Projection::conic;
@@ -334,7 +338,6 @@ void Proj4Projection::full() {
     }
 }
 
-
 void Proj4Projection::wrap(double& x, double& y) {}
 
 void Proj4Projection::corners() {
@@ -356,26 +359,31 @@ void Proj4Projection::corners() {
 void Proj4Projection::centre() {}
 
 PaperPoint Proj4Projection::operator()(const UserPoint& point) const {
-    if (!from_) {
-        from_       = pj_init_plus("+proj=longlat +ellps=WGS84 +datum=WGS84");
+    if (!helper_) {
         projection_ = Epsg::find(*this);
-        to_         = pj_init_plus(projection_->definition());
+        helper_     = new LatLonProjP(projection_->definition());
+
+        if (!helper_->valid()) {
+            // MagLog::error() << pj_strerrno(pj_errno) << endl;
+            MagLog::error() << " proj4 error " << projection_->definition() << endl;
+            ASSERT(false);
+        }
     }
     double x = point.x();
     double y = point.y();
 
-    x *= DEG_TO_RAD;
-    y *= DEG_TO_RAD;
-
-    int error = pj_transform(from_, to_, 1, 1, &x, &y, NULL);
+    int error = helper_->convert(x, y);
     if (error) {
-        MagLog::debug() << pj_strerrno(error) << " for " << point << endl;
-        //		NON Valeurs trop faibles en EPSG:3857 par exemple !  --> Transformation::in(pp) == True !
-        //		-> Avec ces valeurs, si grille globale, les barbules des poles sont tracées dans l'atlantique sud
-        //		en formant une couronne centrée vers( 9°W, 66°35S)
-        //		return PaperPoint(1000000, 10000000);   .// En plus il manque 1 zero !
+        MagLog::debug() << helper_->error(error) << " for " << point << endl;
+        //		NON Valeurs trop faibles en EPSG:3857 par exemple !  -->
+        // Transformation::in(pp) == True !
+        //		-> Avec ces valeurs, si grille globale, les barbules des poles
+        // sont tracées dans l'atlantique sud 		en formant une couronne
+        // centrée vers( 9°W, 66°35S) 		return PaperPoint(1000000,
+        // 10000000);   .// En plus il manque 1 zero !
         return PaperPoint(HUGE_VAL,
-                          HUGE_VAL);  // Devrait suffire pour que Transformation::in(pp) == False qqsoient from_ et to_
+                          HUGE_VAL);  // Devrait suffire pour que Transformation::in(pp) == False
+                                      // qqsoient from_ et to_
     }
 
     return PaperPoint(x, y, point.value_, point.missing(), point.border(), 0, point.name());
@@ -416,25 +424,20 @@ void Proj4Projection::revert(const PaperPoint& xy, UserPoint& point) const {
         return;
     }
 
-
     double x = xy.x();
     double y = xy.y();
 
-    int error = pj_transform(to_, from_, 1, 1, &x, &y, NULL);
+    int error = helper_->revert(x, y);
 
     if (error) {
-        MagLog::debug() << pj_strerrno(error) << endl;
+        MagLog::debug() << helper_->error(error) << endl;
         point = UserPoint(HUGE_VAL, HUGE_VAL);
         return;
     }
 
-    x *= RAD_TO_DEG;
-    y *= RAD_TO_DEG;
-
     point = UserPoint(x, y);
 }
 
-
 bool Proj4Projection::needShiftedCoastlines() const {
     // Will need w new parameter to know!
     return false;
@@ -445,11 +448,10 @@ void Proj4Projection::aspectRatio(double& width, double& height) {
     Transformation::aspectRatio(width, height);
 }
 
-
 void Proj4Projection::add(double lon, double lat) {
-    double x  = lon * DEG_TO_RAD;
-    double y  = DEG_TO_RAD * lat;
-    int error = pj_transform(from_, to_, 1, 1, &x, &y, NULL);
+    double x  = lon;
+    double y  = lat;
+    int error = helper_->convert(x, y);
     userEnveloppe_->push_back(PaperPoint(lon, lat));
     PCEnveloppe_->push_back(PaperPoint(x, y));
 
@@ -462,7 +464,6 @@ void Proj4Projection::add(double lon, double lat) {
     if (y > max_pcy_)
         max_pcy_ = y;
 
-
     if (lon < gridMinLon_)
         gridMinLon_ = lon;
     if (lat < gridMinLat_)
@@ -489,7 +490,6 @@ void Proj4Projection::conic() {
     add(projection_->maxlon_, projection_->maxlat_);
 }
 
-
 void Proj4Projection::simple() {
     userEnveloppe_->clear();
     PCEnveloppe_->clear();
@@ -522,19 +522,13 @@ void Proj4Projection::projectionSimple() {
     min_pcy_ = min_latitude_;
     max_pcx_ = max_longitude_;
     max_pcy_ = max_latitude_;
-    pj_transform(to_, from_, 1, 1, &min_longitude_, &min_latitude_, NULL);
-    pj_transform(to_, from_, 1, 1, &max_longitude_, &max_latitude_, NULL);
+    helper_->revert(min_longitude_, min_latitude_);
+    helper_->revert(max_longitude_, max_latitude_);
 
-    min_longitude_ *= RAD_TO_DEG;
-    min_latitude_ *= RAD_TO_DEG;
-    max_longitude_ *= RAD_TO_DEG;
     if (max_longitude_ < 0) {
         max_longitude_ += 360.;
     }
 
-    max_latitude_ *= RAD_TO_DEG;
-
-
     magics::Polyline box;
     box.box(PaperPoint(min_pcx_, min_pcy_), PaperPoint(max_pcx_, max_pcy_));
 
@@ -551,7 +545,6 @@ void Proj4Projection::projectionSimple() {
     coordinates_system_ = "latlon";
 }
 
-
 void Proj4Projection::geos() {
     userEnveloppe_->clear();
     PCEnveloppe_->clear();
@@ -561,16 +554,15 @@ void Proj4Projection::geos() {
     max_pcx_ = -DBL_MAX;
     max_pcy_ = -DBL_MAX;
 
-
     map<double, vector<double> > helper;
 
     for (int lat = projection_->minlat_; lat <= projection_->maxlat_; lat++) {
         helper.insert(make_pair(lat, vector<double>()));
 
         for (int lon = projection_->minlon_; lon <= projection_->maxlon_; lon++) {
-            double x  = lon * DEG_TO_RAD;
-            double y  = DEG_TO_RAD * lat;
-            int error = pj_transform(from_, to_, 1, 1, &x, &y, NULL);
+            double x  = lon;
+            double y  = lat;
+            int error = helper_->convert(x, y);
 
             if (!error) {
                 helper[lat].push_back(lon);
@@ -578,7 +570,6 @@ void Proj4Projection::geos() {
         }
     }
 
-
     // now we create the envelope...
     for (map<double, vector<double> >::iterator lat = helper.begin(); lat != helper.end(); ++lat) {
         if (lat->second.empty())
@@ -609,16 +600,7 @@ void Proj4Projection::geos() {
 magics::Polyline& Proj4Projection::getSimplePCBoundingBox() const {
     static magics::Polyline box;
     box.box(PaperPoint(min_pcx_, min_pcy_), PaperPoint(max_pcx_, max_pcy_));
-    /*
-    vector<magics::Polyline*> newbox;
-    PCEnveloppe_->intersect(box, newbox);
-    if (newbox.empty()) {
-        MagLog::warning() << "Proj4 : the sub-area is not valid : use global view instead" << endl;
-    }
-    else {
-        PCEnveloppe_ = newbox.front();
-    }
-    */
+
     return box;
 }
 
@@ -631,7 +613,6 @@ void Proj4Projection::tpers() {
     max_pcx_ = -DBL_MAX;
     max_pcy_ = -DBL_MAX;
 
-
     map<double, vector<double> > helper;
 
     // projection_view_latitude_
@@ -643,9 +624,9 @@ void Proj4Projection::tpers() {
     for (double lat = projection_view_latitude_; lat <= 90; lat += step) {
         lastlon = missing;
         for (double lon = projection_view_longitude_; lon <= projection_view_longitude_ + 360; lon += step) {
-            double x  = lon * DEG_TO_RAD;
-            double y  = DEG_TO_RAD * lat;
-            int error = pj_transform(from_, to_, 1, 1, &x, &y, NULL);
+            double x  = lon;
+            double y  = lat;
+            int error = helper_->convert(x, y);
             if (error) {
                 // we reach a border!
 
@@ -654,15 +635,16 @@ void Proj4Projection::tpers() {
                 }
                 break;
             }
+
             lastlon = lon;
         }
     }
     for (double lat = 90; lat >= projection_view_latitude_; lat -= step) {
         lastlon = missing;
         for (double lon = projection_view_longitude_; lon >= projection_view_longitude_ - 360; lon -= step) {
-            double x  = lon * DEG_TO_RAD;
-            double y  = DEG_TO_RAD * lat;
-            int error = pj_transform(from_, to_, 1, 1, &x, &y, NULL);
+            double x  = lon;
+            double y  = lat;
+            int error = helper_->convert(x, y);
             if (error) {
                 // we reach a border!
                 if (lastlon != missing) {
@@ -670,34 +652,34 @@ void Proj4Projection::tpers() {
                 }
                 break;
             }
+
             lastlon = lon;
         }
     }
 
-
     for (double lat = projection_view_latitude_; lat >= -90; lat -= step) {
         lastlon = missing;
         for (double lon = projection_view_longitude_; lon >= projection_view_longitude_ - 360; lon -= step) {
-            double x  = lon * DEG_TO_RAD;
-            double y  = DEG_TO_RAD * lat;
-            int error = pj_transform(from_, to_, 1, 1, &x, &y, NULL);
+            double x  = lon;
+            double y  = lat;
+            int error = helper_->convert(x, y);
             if (error) {
                 // we reach a border!
-
                 if (lastlon != missing) {
                     add(lastlon, lat);
                 }
                 break;
             }
+
             lastlon = lon;
         }
     }
     for (double lat = -90; lat <= projection_view_latitude_; lat += step) {
         lastlon = missing;
         for (double lon = projection_view_longitude_; lon <= projection_view_longitude_ + 360; lon += step) {
-            double x  = lon * DEG_TO_RAD;
-            double y  = DEG_TO_RAD * lat;
-            int error = pj_transform(from_, to_, 1, 1, &x, &y, NULL);
+            double x  = lon;
+            double y  = lat;
+            int error = helper_->convert(x, y);
             if (error) {
                 // we reach a border!
                 if (lastlon != missing) {
@@ -705,6 +687,7 @@ void Proj4Projection::tpers() {
                 }
                 break;
             }
+
             lastlon = lon;
         }
     }
@@ -713,13 +696,10 @@ void Proj4Projection::tpers() {
     PCEnveloppe_->push_back(PCEnveloppe_->front());
 }
 
-
 void Proj4Projection::boundingBox(double& xmin, double& ymin, double& xmax, double& ymax) const {
-    if (!from_) {
-        from_ = pj_init_plus("+proj=longlat +ellps=WGS84 +datum=WGS84");
-
+    if (!helper_) {
         projection_ = Epsg::find(definition_);
-        to_         = pj_init_plus(projection_->definition());
+        helper_     = new LatLonProjP(projection_->definition());
     }
 
     ymin = gridMinLat_;
@@ -787,7 +767,6 @@ void Proj4Projection::fill(double& width, double& height) {
 void Proj4Projection::gridLongitudes(const GridPlotting& grid) const {
     vector<double> longitudes = grid.longitudes();
 
-
     const double step = 0.5;
     // longitudes.push_back(180);
     const vector<double>::const_iterator lon_end = longitudes.end();
@@ -811,11 +790,9 @@ void Proj4Projection::gridLongitudes(const GridPlotting& grid) const {
             }
         }
 
-
         grid.add(poly);
     }
 
-
     grid.addFrame(*PCEnveloppe_);
 }
 
@@ -843,7 +820,6 @@ void Proj4Projection::gridLatitudes(const GridPlotting& grid) const {
             }
         }
 
-
         grid.add(poly);
     }
 
@@ -882,7 +858,6 @@ void Proj4Projection::labels(const LabelPlotting& label, DrawingVisitor& visitor
     }
 }
 
-
 void Proj4Projection::labels(const LabelPlotting& label, LeftAxisVisitor& visitor) const {
     if (projection_->method_ == "simple") {
         const vector<double>& latitudes = label.latitudes();
@@ -1111,15 +1086,15 @@ void Proj4Projection::reprojectSpeedDirection(const PaperPoint& point, pair<doub
     double x = point.x_;
     double y = point.y_;
 
-    double u = x + (sin(wind.second * DEG_TO_RAD));
-    double v = y + (cos(wind.second * DEG_TO_RAD));
+    double u = x + (sin(RAD(wind.second)));
+    double v = y + (cos(RAD(wind.second)));
 
     fast_reproject(x, y);
     fast_reproject(u, v);
 
     double rotation = atan2((u - x), (v - y));
 
-    wind.second = (rotation * RAD_TO_DEG);
+    wind.second = DEG(rotation);
 }
 
 void Proj4Projection::revert(const vector<std::pair<double, double> >& in,
@@ -1136,19 +1111,19 @@ void Proj4Projection::revert(const vector<std::pair<double, double> >& in,
             continue;
         }
 
-        int error = pj_transform(to_, from_, 1, 1, &x, &y, NULL);
+        int error = helper_->revert(x, y);
 
         if (error) {
-            MagLog::error() << pj_strerrno(error) << " for " << pt->first << " " << pt->second << endl;
+            MagLog::error() << helper_->error(error) << " for " << pt->first << " " << pt->second << endl;
             out.push_back(make_pair(HUGE_VAL, HUGE_VAL));
         }
         else {
-            double lon = x * RAD_TO_DEG;
+            double lon = x;
             if (lon > gridMaxLon_)
                 lon -= 360.;
             else if (lon < gridMinLon_)
                 lon += 360.;
-            double lat = y * RAD_TO_DEG;
+            double lat = y;
             out.push_back(make_pair(lon, lat));
         }
     }
@@ -1180,9 +1155,11 @@ void Proj4Projection::coastSetting(map<string, string>& setting, double abswidth
     setting["coast"]      = resol + "/ne_" + resol + "_coastline";
     setting["rivers"]     = resol + "/ne_" + resol + "_rivers_lake_centerlines";
     setting["boundaries"] = resol + "/ne_" + resol + "_admin_0_boundary_lines_land";
-    //    setting["administrative_boundaries"] = resol + "/ne_" + resol + "_admin_1_states_provinces";
+    //    setting["administrative_boundaries"] = resol + "/ne_" + resol +
+    //    "_admin_1_states_provinces";
 
-    //! \note Administraive borders hardcoded to 10m resolution (low res version do not contain all info)
+    //! \note Administraive borders hardcoded to 10m resolution (low res version
+    //! do not contain all info)
     setting["administrative_boundaries"] = "10m/ne_10m_admin_1_states_provinces_lines";
 }
 
@@ -1227,14 +1204,13 @@ bool Proj4Projection::fast_reproject(double& x, double& y) const {
         x -= (factor - width_);
     }
 
-    x *= DEG_TO_RAD;
-    y *= DEG_TO_RAD;
-    int error = pj_transform(from_, to_, 1, 1, &x, &y, NULL);
+    int error = helper_->convert(x, y);
 
     if (error) {
         x = HUGE_VAL;
         y = HUGE_VAL;
-        // MagLog::warning()  << pj_strerrno(error) << " for " << x << " " << y << endl;
+        // MagLog::warning()  << pj_strerrno(error) << " for " << x << " " << y <<
+        // endl;
         return false;
     }
     if (wraparound_)
@@ -1283,13 +1259,11 @@ void Proj4Projection::setDefinition(const string& json) {
     set(node);
 }
 
-
 void Proj4Automatic::setMinMaxX(double min, double max) {
     min_longitude_ = min;
     max_longitude_ = max;
 }
 
-
 void Proj4Automatic::setMinMaxY(double min, double max) {
     min_latitude_ = min;
     max_latitude_ = max;
@@ -1319,7 +1293,6 @@ void Proj4Automatic::setMinMaxY(double min, double max) {
     init_ = true;
     fill(width_, height_);
 
-
     // Now apply teh aspect ratio and reintialise!
 
     min_latitude_  = gridMinLat_;
@@ -1327,7 +1300,6 @@ void Proj4Automatic::setMinMaxY(double min, double max) {
     max_latitude_  = gridMaxLat_;
     max_longitude_ = gridMaxLon_;
 
-
     init();
 }
 
@@ -1343,19 +1315,18 @@ void Proj4Automatic::aspectRatio(double& width, double& height) {
     height_ = height;
 }
 
-
 void Proj4Automatic::setNewPCBox(double minx, double miny, double maxx, double maxy) {
-    double x  = minx;
-    double y  = miny;
-    int error = pj_transform(to_, from_, 1, 1, &x, &y, NULL);
+    gridMinLon_ = minx;
+    gridMinLat_ = miny;
+    int error   = helper_->revert(gridMinLon_, gridMinLat_);
+    if (error) {
+        MagLog::error() << "Error Proj4Automatic::setNewPCBox to check " << endl;
+    }
 
-    gridMinLon_ = x * RAD_TO_DEG;
-    gridMinLat_ = y * RAD_TO_DEG;
-
-    x     = maxx;
-    y     = maxy;
-    error = pj_transform(to_, from_, 1, 1, &x, &y, NULL);
-
-    gridMaxLon_ = x * RAD_TO_DEG;
-    gridMaxLat_ = y * RAD_TO_DEG;
+    gridMaxLon_ = maxx;
+    gridMaxLat_ = maxy;
+    error       = helper_->revert(gridMaxLon_, gridMaxLat_);
+    if (error) {
+        MagLog::error() << "Error Proj4Automatic::setNewPCBox to check " << endl;
+    }
 }
