@@ -61,18 +61,10 @@ void IsoPlot::print(ostream& out) const {
 class CellBox : public AutoVector<CellBox> {
 public:
     CellBox(const CellArray* parent, int row1, int row2, int column1, int column2) :
-        parent_(parent),
-        row1_(row1),
-        row2_(row2),
-        column1_(column1),
-        column2_(column2) {}
+        parent_(parent), row1_(row1), row2_(row2), column1_(column1), column2_(column2) {}
 
     CellBox(const CellArray* parent) :
-        parent_(parent),
-        row1_(0),
-        row2_(parent->rows_ - 1),
-        column1_(0),
-        column2_(parent->columns_ - 1) {}
+        parent_(parent), row1_(0), row2_(parent->rows_ - 1), column1_(0), column2_(parent->columns_ - 1) {}
 
     CellBox() : parent_(0), row1_(0), row2_(0), column1_(0), column2_(0) {}
 
@@ -1604,8 +1596,7 @@ void IsoPlot::visit(Data& data, PointsHandler& points, HistoVisitor& visitor) {
 
 CellArray::CellArray(MatrixHandler& data, IntervalMap<int>& range, const Transformation& transformation, int width,
                      int height, float resol, const string& technique) :
-    rangeFinder_(range),
-    data_(data) {
+    rangeFinder_(range), data_(data) {
     Timer timer("CellArray", "CellArray");
     int r = height / resol;
     int c = width / resol;
@@ -1720,8 +1711,7 @@ GridArray::GridArray(MatrixHandler& data, IntervalMap<int>& range, const Transfo
 
 GridCell::GridCell(const CellArray& data, int row, int column, const Transformation& transformation,
                    const string& technique) :
-    Cell(data),
-    transformation_(transformation) {
+    Cell(data), transformation_(transformation) {
     row_     = row;
     column_  = column;
     int row1 = (row) ? row - 1 : 0;
